@@ -399,6 +399,22 @@ class TestExecution:
             if i>=3:
                 assert not numdir.new(ext=str(i-3)).check()
 
+    def test_make_numbered_dir_case_insensitive(self, tmpdir, monkeypatch):
+        # https://github.com/pytest-dev/pytest/issues/708
+        monkeypatch.setattr(py._path.local, 'normcase', lambda path: path.lower())
+        monkeypatch.setattr(tmpdir, 'listdir', lambda: [tmpdir._fastjoin('case.0')])
+        numdir = local.make_numbered_dir(prefix='CAse.', rootdir=tmpdir,
+                                         keep=2, lock_timeout=0)
+        assert numdir.basename.endswith('.1')
+
+    def test_make_numbered_dir_case_sensitive(self, tmpdir, monkeypatch):
+        # https://github.com/pytest-dev/pytest/issues/708
+        monkeypatch.setattr(py._path.local, 'normcase', lambda path: path)
+        monkeypatch.setattr(tmpdir, 'listdir', lambda: [tmpdir._fastjoin('case.0')])
+        numdir = local.make_numbered_dir(prefix='CAse.', rootdir=tmpdir,
+                                         keep=2, lock_timeout=0)
+        assert numdir.basename.endswith('.0')
+
     def test_make_numbered_dir_NotImplemented_Error(self, tmpdir, monkeypatch):
         def notimpl(x, y):
             raise NotImplementedError(42)
